"""Command for sending the newsletter"""

import sys

from optparse import make_option

from django.conf import settings
from django.utils.translation import activate
from django.core.management.base import BaseCommand

from jmb.mailer.sender import Mailer
from jmb.mailer.models import EmailObject


class Command(BaseCommand):
    """Send the newsletter in queue"""
    help = 'Send mails in queue'

    option_list = BaseCommand.option_list + (
        make_option('-c', '--set-canceled', action='store_true', default=False,
                    dest='set_canceled', help='Set all emails in error when creating Mailer object, as cancelled'),
        )
    
    def handle(self, **options):
        verbose = int(options['verbosity'])

        if verbose:
            print 'Starting sending emails...'

        activate(settings.LANGUAGE_CODE)

        for email in EmailObject.objects.exclude(
            status=settings.MAILER_EMAIL_STATUS_SENT
        ).exclude(
            status=settings.MAILER_EMAIL_STATUS_DRAFT
        ).exclude(
            status=settings.MAILER_EMAIL_STATUS_CANCELED
        ):
            try:
                mailer = Mailer(email, verbose=verbose)
            except Exception, e:
                print 'Problem emailing %s: %s - %s' % (email.email_address, email.subject, e)
                if options.get('set_canceled'):
                    email.status = settings.MAILER_EMAIL_STATUS_CANCELED
                    email.save()
                continue
            if mailer.can_send:
                if verbose:
                    print u'Start emailing %s: %s' % (email.email_address, email.subject)
                mailer.send()

        if verbose:
            print 'End session sending'
